﻿using HarmonyLib;
using Steamworks;
using System;
using System.Linq;
using System.Reflection;
using TMPro;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;

public class ServerStorage : Mod
{
    Harmony HarmonyInstance;

    public static Material RedCable;
    public static Material BlueCable;

    public void Start()
    {
        HarmonyInstance = new Harmony("fr.tekgamer.serverstorage");
        HarmonyInstance.PatchAll(Assembly.GetExecutingAssembly());
        AssetBundle bundle = AssetBundle.LoadFromMemory(GetEmbeddedFileBytes("serverstorage.assets"));

        RedCable = bundle.LoadAsset<Material>("CableRed");
        BlueCable = bundle.LoadAsset<Material>("CableBlue");

        Item_Base storage = bundle.LoadAsset<Item_Base>("ItemBase_Placeable_ServerStorage");
        Storage_Small block = storage.settings_buildable.GetBlockPrefab(0) as Storage_Small;
        block.gameObject.AddComponent<ServerStorageManager>();
        RAPI.RegisterItem(storage);
        RAPI.AddItemToBlockQuadType(storage, RBlockQuadType.quad_floor);
        RAPI.AddItemToBlockQuadType(storage, RBlockQuadType.quad_foundation);

        Debug.Log("Mod ServerStorage has been loaded!");
    }

    public void OnModUnload()
    {
        Debug.Log("Mod ServerStorage has been unloaded!");
    }
}

public class ServerStorageManager : MonoBehaviour
{
    Inventory inventory;
    Storage_Small block;
    InputField field;
    TextMeshPro text;
    Battery battery;

    DateTime lastDecrease;
    int lossPerTime = 1;
    float amountPerMinute = 0.9f;

    LineRenderer battery_plus;
    LineRenderer battery_minus;

    void OnDestroy()
    {
        if (Semih_Network.IsHost && block != null && block.GetInventoryReference() != null)
        {
            block.GetInventoryReference().allSlots.ForEach(slot =>
            {
                if (slot.HasValidItemInstance())
                {
                    Helper.DropItem(slot.itemInstance, transform.position, Vector3.zero, true);
                }
            });
        }
    }

    public bool CanBeOpened = false;
    public void Start()
    {
        block = GetComponent<Storage_Small>();
        inventory = block?.GetInventoryReference();
        text = block?.GetComponentInChildren<TextMeshPro>();
        field = inventory?.GetComponentInChildren<InputField>();
        field?.onValueChanged?.RemoveAllListeners();
        field?.onValueChanged?.AddListener(OnSearchValueChanged);
    }

    public static Color GetValueColor(float value)
    {
        if (value <= 25)
        {
            return new Color(189 / 255f, 38 / 255f, 38 / 255f, 1);
        }
        else if (value <= 50)
        {
            return new Color(227 / 255f, 227 / 255f, 18 / 255f, 1);
        }
        else if (value <= 75)
        {
            return new Color(224 / 255f, 164 / 255f, 52 / 255f, 1);
        }
        else
        {
            return new Color(72 / 255f, 176 / 255f, 79 / 255f, 1);
        }
    }

    public Battery GetClosestBattery()
    {
        Battery closestBattery = FindObjectsOfType<Battery>().Where(b => !b.BatterySlotIsEmpty).OrderBy(t => (t.transform.position - transform.position).sqrMagnitude).FirstOrDefault() as Battery;
        if (closestBattery && Vector3.Distance(closestBattery.transform.position, transform.position) < 5)
        {
            return closestBattery;
        }
        else
        {
            return null;
        }
    }

    void Update()
    {
        if (block && field && block.IsOpen)
        {
            if ((Network_Player)Traverse.Create(block).Field("currentPlayer").GetValue() == RAPI.GetLocalPlayer())
            {
                if (field.isFocused || EventSystem.current.currentSelectedGameObject == field.gameObject)
                {
                    CanvasHelper.ActiveMenu = MenuType.PauseMenu;
                }
                else
                {
                    CanvasHelper.ActiveMenu = MenuType.Inventory;
                }
            }
        }

        if (block != null && inventory != null && StorageManager.allStorages.Contains(block))
        {
            // Is Valid & Placed.
            if (battery == null)
            {
                if (battery_plus != null) { Destroy(battery_plus.gameObject); };
                if (battery_minus != null) { Destroy(battery_minus.gameObject); };

                battery = GetClosestBattery();
                if (battery != null)
                {
                    battery_plus = CreateLine(transform.Find("ConsolePrefab").position, battery.transform.position, false, battery.gameObject, gameObject);
                    battery_minus = CreateLine(transform.Find("ConsolePrefab").position + new Vector3(0.02f, 0, 0), battery.transform.position + new Vector3(0.02f, 0, 0), true, battery.gameObject, gameObject);
                }
                else
                {
                    text.text = "NO BATTERY\nFOUND";
                    text.color = Color.red;
                }
                CanBeOpened = false;
                return;
            }
            if (battery.BatterySlotIsEmpty || battery.GetBatteryInstance() == null)
            {
                text.text = "NO BATTERY\nFOUND";
                text.color = Color.red;
                CanBeOpened = false;
            }
            else
            {
                ItemInstance instance = battery.GetBatteryInstance();
                if (instance != null)
                {
                    if (Semih_Network.IsHost)
                    {
                        if (lastDecrease.AddMinutes(amountPerMinute) < DateTime.Now)
                        {
                            lastDecrease = DateTime.Now;
                            battery.RemoveBatteryUsesNetworked(lossPerTime);
                        }
                    }
                    float percentage = instance.NormalizedUses * 100;
                    CanBeOpened = percentage > 0;
                    text.text = "<color=white>BATTERY :\n</color>" + Math.Round(percentage) + "%";
                    text.color = GetValueColor(percentage);
                }
            }
            if (!block.IsOpen && !string.IsNullOrWhiteSpace(field.text))
            {
                field.text = string.Empty;
                inventory.allSlots.ForEach(slot =>
                {
                    slot.gameObject.SetActive(true);
                });
            }
        }
        else
        {
            Start();
        }
    }

    LineRenderer CreateLine(Vector3 start, Vector3 end, bool blue, GameObject battery, GameObject storage)
    {
        LineRenderer lineRenderer = new GameObject("Line").AddComponent<LineRenderer>();
        lineRenderer.material = blue ? ServerStorage.BlueCable : ServerStorage.RedCable;
        lineRenderer.startWidth = 0.01f;
        lineRenderer.endWidth = 0.01f;
        lineRenderer.positionCount = 2;
        lineRenderer.useWorldSpace = true;

        lineRenderer.SetPosition(0, start);
        lineRenderer.SetPosition(1, end);
        LineDependsOnGameObject t = lineRenderer.gameObject.AddComponent<LineDependsOnGameObject>();
        t.first = battery;
        t.second = storage;
        t.initialized = true;
        return lineRenderer;
    }

    public void OnSearchValueChanged(string value)
    {
        inventory.allSlots.ForEach(slot =>
        {
            if (string.IsNullOrWhiteSpace(value))
            {
                slot.gameObject.SetActive(true);
                return;
            }
            if (slot.HasValidItemInstance() && !slot.IsEmpty)
            {
                if (slot.itemInstance != null && (slot.itemInstance.UniqueName.ToLower().Contains(value.ToLower()) || slot.itemInstance.settings_Inventory.DisplayName.ToLower().Contains(value.ToLower())))
                {
                    slot.gameObject.SetActive(true);
                }
                else
                {
                    slot.gameObject.SetActive(false);
                }
            }
            else
            {
                slot.gameObject.SetActive(false);
            }
        });
    }
}

public class LineDependsOnGameObject : MonoBehaviour
{
    public GameObject first;
    public GameObject second;
    public bool initialized = false;

    public void Update()
    {
        if (!initialized) { return; }
        if (first == null || second == null)
        {
            Destroy(gameObject);
        }
    }
}

[HarmonyPatch(typeof(Storage_Small))]
[HarmonyPatch("OnIsRayed")]
public static class HarmonyPatch_OpenStorage
{
    private static bool Prefix(Storage_Small __instance)
    {
        if (__instance.itemToReturnOnDestroy.UniqueName == "Placeable_ServerStorage")
        {
            ServerStorageManager storage = __instance.GetComponent<ServerStorageManager>();
            if (storage.CanBeOpened)
            {
                return true;
            }
            else
            {
                ComponentManager<CanvasHelper>.Value.displayTextManager.ShowText("The server storage needs power to be used!");
                return false;
            }

        }
        return true;
    }
}